/*  Contains VRekrer SCPI functions
 *   Callbacks for 18b DAC
*/

#include "Vrekrer_scpi_parser.h"

void initScpi(void)
{
  // Set up SCPI
  my_instrument.RegisterCommand(F("*IDN?"), &Identify);           // F() specifies Flash strings
  my_instrument.RegisterCommand(F("*RST"), &Reset);
  my_instrument.SetCommandTreeBase(F("MEASure"));
    my_instrument.RegisterCommand(F(":CURRent?"), &GetCurrent);
    my_instrument.RegisterCommand(F(":VOLTage?"), &GetVoltage);
  my_instrument.SetCommandTreeBase(F("SOURce"));
    my_instrument.RegisterCommand(F(":VOLTage"), &SetVoltage);
    my_instrument.RegisterCommand(F(":VOLTage?"), &GetSourceVoltage);
}


// SCPI Commands
/*
 * Respond to *IDN?
 */
void Identify(SCPI_C commands, SCPI_P parameters, Stream& interface) {
  interface.println(F("Erickson Engineering,18b DAC,#01,v2.0"));
}

void Reset(SCPI_C commands, SCPI_P parameters, Stream& interface) {
  interface.println(F("Doing a reset"));
  Iset = 0L;  
  setDac();
}


/**
 * Read the DAC Set voltage: Iset
 */
void GetSourceVoltage(SCPI_C commands, SCPI_P parameters, Stream& interface) {
  interface.print((float)Iset / 1000000L,5);
  Serial.println(" V");
}


void GetCurrent(SCPI_C commands, SCPI_P parameters, Stream& interface) {
  interface.println(String(current, 12));
}

void GetVoltage(SCPI_C commands, SCPI_P parameters, Stream& interface) {
  interface.println(String(voltage, 9));
}

void SetVoltage(SCPI_C commands, SCPI_P parameters, Stream& interface) {
  // For simplicity no bad parameter check is done.
  if (parameters.Size() > 0) {
    voltage = String(parameters[0]).toFloat();
//  interface.println(String(parameters[0]));
    voltage = constrain(voltage, 0.0, 10.4);
    Iset = voltage * 1000000L;
    setDac();

    
  }
}
